clear all
close all
%--------------------------------Power-law data---------------------------------
% Power-law data
    b=0.434;
    N0=100;
    V0=0.001;
    a=N0*V0^b;
% Power-law simulation parameters
    vol_min=.001;
    vol_max=100000;
    Fmax=a*vol_min^-b;
    Fmin=a*vol_max^-b;
%-----------------------------Number of simulations --------------------------
    N_sim=1000000;
%enterval beween plot
    delta_plot= 10000;
%-----------------------------Number of events per year --------------------------
% Distribution of Poisson simulation    
    lambda=N0;   
    cum_poisson=rand(N_sim,1);
    X = poissinv(cum_poisson,lambda);
   
%Rank for power law
    N=N0:-1:1;
%-----------------------------Uniform distributions parameters   --------------------------
% Number of vehicles Nv    
    Nv=5000;
% Range of number of vehicles Nv+- dNv/2
    dNv=1000;
% Velocity of the vehicles 
    Vv=	80;
    dVv=45;
% Additional passenger 0 or 1
    dPas=0;
% lenght of the vehicles 
    Lv=5.4;
%------------------------------- Plot of the power-law ------------------------------
% 
    V=-3:1:4;
    Vol=10.^V;
    F=a*Vol.^-b;
    subplot(2,2,1)
    loglog(Vol,F,'r','LineWidth',2)
    grid on
    grid minor
    box on
    xlim([1e-3 1e4])
%------------------------------- Variable declaration ------------------------------
% simulated frequency
    Fsim=1;
% simulated volume   
    vol=1;
% Probability of propagation      
    Pp=1;
% Vulnerability   
    Vuln=1;
% Exposition    
    Exp1=1;
% Risk
    R=1;

%-------------------------------------- Simulations-------------------------------------
% Initialize arrays    
    risk = zeros(1,N_sim);
    plotD= cell(N_sim,1) ;
    tic;
% Action
    parfor i=1:N_sim
      % Simulation of the proability and simulation of the volume vol
        dim = X(i,1);
        Fsim=Fmin+rand(dim,1)*(Fmax-Fmin);
        vol=sort((Fsim/a).^(-1/b),'ascend');
      % Rank for power law
        plotD{i}=[vol,(dim:-1:1)'];
      % Debris width
        D=(vol.^(1/3)).*(0.5+rand(dim,1));
      % Replace the smaller volumes       
        vol(vol<0.01)=0.01;
      % Computation of the prorbability of propagation depending on the
      % volume
        Pp         = 0.180*(log10(vol)-.5+rand(dim,1))+0.46;   
        Pp(Pp<0.1) = 0.1;
        Pp(Pp>1.0) = 1.0;
      % Calculation of the vulnerability  
        rVuln  = (log10(vol)-0.5+rand(dim,1));
        Vuln = 0.038000*rVuln.^2 +0.152000*rVuln+0.202000; 
        Vuln(Vuln<0.05)=0.05;
        Vuln(log10(vol)>3)=1.0;
      % Velocity of the vehicles randomized
        Vv2=(Vv-0.5*dVv+dVv*rand(dim,1));
      % Number of the vehicles randomized
        Nv1=(Nv-0.5*dNv+dNv*rand(dim,1));
      % Velocity of the exposition
        Exp1=Nv1.*(Lv+D)./(Vv2.*1000*24);
      % Randomized 1 or 2 passengers 
        E=1+dPas*round(rand(dim,1));
     % Risk calculation
        R=Exp1.*Pp.*Vuln.*E;%/X(i,1);
     % Sum of the simulated events
        risk(i)=sum(R);
    end
    toc
    
%-------------------------------------- Figures-------------------------------------
figure(1)
    subplot(2,2,1)
% Plot of selected volume ditributions
    for i = 1:N_sim
        if mod(i,delta_plot)==0
            hold on
            plot(plotD{i}(:,1),plotD{i}(:,2),'-+k')
        end
    end
    box on
    ylim([1 1000])
    xlim([.001 10000])
    title('a. Realizations of volume distributions');
    loglog(Vol,F,'r','LineWidth',2)
    hold off
    xlabel('Volume [m^3]')
    ylabel('Number of events \geq N');
    legend({'Original distribution','Simulations'},'Location','northeast');
% Histogram of risk simulations
    subplot(2,2,2)
    histogram(risk,50)
    box on
    xlabel('Number of fatalities [yr^{-1}]')
    ylabel('Simulation frequency [#]');
    title('b. Histogram of simulation averages');
%Exceedance curves of the frequence of deaths    
    risk=sort(risk,'ascend');
    subplot(2,2,3)
    N100=1:1:N_sim;
    plot(risk,1-N100/N_sim,'r','LineWidth',2)
    box on
    xlabel('Frequency N of fatalities [yr^{-1}]')
    ylabel('Probability that n \geq N');
    title('c. Exceedance curve for frequency of fatalities');
%Exceedance curves of the return period of death 
    subplot(2,2,4)
    box on
    plot(1./risk,1-N100/N_sim,'LineWidth',2)
    xlabel('Return period for fatal accidents T [yr^{-1}]')
    ylabel('Probability that the return period t\leq T');
    title('d. Exceedance curve for return period of fatalities');
% Calculation of quantiles
    m(1)=mean(risk);  
    m(2)=min(risk);
    m(3)=quantile(risk,0.025);
    m(4)=quantile(risk,0.05);
    m(5)=median(risk);
    m(6)=quantile(risk,0.95);
    m(7)=quantile(risk,0.975);
    m(8)=max(risk);
% Plot of the Poisson distribution
figure(2)
    X=sort(X,'ascend');
    plot(X,N100,'-b','LineWidth',2);
    xlabel('Number of rockfalls per year');
    ylabel('Cumulative number of simulations');
% Print the quantiles for the probabitility
    prob=round(m',3);
    fprintf('\nQuantiles for the probability:')
    fprintf('\n------------------------------\n');
    disp(prob);
% Print the quantiles for the return period
    T=round(1./m',1);
    fprintf('\nQuantiles for the return period:');
    fprintf('\n-------------------------------\n');
    disp(T);
